"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.knex = void 0;
exports.setupDatabase = setupDatabase;
exports.updateDatabase = updateDatabase;
const node_path_1 = __importDefault(require("node:path"));
const node_fs_1 = __importDefault(require("node:fs"));
const main_1 = require("./main");
let rawDb;
function setupDatabase() {
    return __awaiter(this, void 0, void 0, function* () {
        exports.knex = require("knex")({
            client: 'better-sqlite3',
            connection: {
                filename: node_path_1.default.join(main_1.bot.dirname, '../bot.db')
            },
            useNullAsDefault: true,
            pool: {
                afterCreate: (conn, done) => {
                    rawDb = conn;
                    conn.pragma('journal_mode = WAL');
                    done(null, conn);
                }
            }
        });
    });
}
function updateDatabase() {
    return __awaiter(this, void 0, void 0, function* () {
        const userVersion = (yield exports.knex.raw("PRAGMA user_version;"))[0].user_version;
        const migrateFolder = node_path_1.default.join(main_1.bot.dirname, "migrate");
        if (!node_fs_1.default.existsSync(migrateFolder))
            return;
        let updating = true;
        let nextVersion = userVersion;
        while (updating) {
            nextVersion = nextVersion + 1;
            const nextVersionFile = node_path_1.default.join(migrateFolder, `${nextVersion}.sql`);
            if (!node_fs_1.default.existsSync(nextVersionFile)) {
                updating = false;
                break;
            }
            yield rawDb.exec(node_fs_1.default.readFileSync(nextVersionFile, { encoding: "utf-8" }));
            rawDb.pragma(`user_version = ${nextVersion}`);
        }
    });
}
