"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.bot = exports.Bot = exports.knex = void 0;
// IMPORTS
const dotenv_1 = __importDefault(require("dotenv"));
dotenv_1.default.config();
const node_fs_1 = __importDefault(require("node:fs"));
const node_path_1 = __importDefault(require("node:path"));
const discord_js_1 = require("discord.js");
const log_1 = require("./log");
const db_1 = require("./db");
var db_2 = require("./db");
Object.defineProperty(exports, "knex", { enumerable: true, get: function () { return db_2.knex; } });
class Bot {
    setup(dirname) {
        return __awaiter(this, void 0, void 0, function* () {
            this.dirname = dirname;
            yield (0, db_1.setupDatabase)();
            yield (0, db_1.updateDatabase)();
        });
    }
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            this.log = new log_1.Log();
            if (!this.dirname) {
                this.log.error(`Bot was not setup.`);
                process.exit();
            }
            this.log.info("Press Control+C to stop the bot");
            // ENVIROMENT VARS
            if (!node_fs_1.default.existsSync(".env")) {
                this.log.error(`No .env file is in the directory. Please add one`);
                process.exit();
            }
            ;
            const botToken = process.env.DISCORD_TOKEN;
            if (botToken == undefined) {
                this.log.error(`The \"DISCORD_TOKEN\" wasn't found in the .env file.\nIt can be added with: \"DISCORD_TOKEN=mytokenhere\"`);
                process.exit();
            }
            ;
            // CREATE CLIENT
            this.client = new discord_js_1.Client({ intents: [
                    discord_js_1.GatewayIntentBits.Guilds,
                    discord_js_1.GatewayIntentBits.GuildMessages,
                    discord_js_1.GatewayIntentBits.MessageContent
                ] });
            // IMPORT COMMANDS
            this.commands = new discord_js_1.Collection();
            this.commandsArray = [];
            const foldersPath = node_path_1.default.join(this.dirname, 'commands');
            if (node_fs_1.default.existsSync(foldersPath)) {
                const commandFolders = node_fs_1.default.readdirSync(foldersPath);
                for (const folder of commandFolders) {
                    const commandsPath = node_path_1.default.join(foldersPath, folder);
                    let commandFiles;
                    if (node_fs_1.default.statSync(commandsPath).isDirectory()) {
                        commandFiles = node_fs_1.default.readdirSync(commandsPath).filter((file) => file.endsWith('.js'));
                    }
                    else {
                        commandFiles = [""];
                    }
                    for (const file of commandFiles) {
                        const filePath = node_path_1.default.join(commandsPath, file);
                        const command = require(filePath);
                        if ('data' in command && 'execute' in command) {
                            this.commands.set(command.data.name, command);
                            this.commandsArray.push(command.data.toJSON());
                        }
                        else {
                            this.log.warn(`The command at ${filePath} is missing a required "data" or "execute" property.`);
                        }
                        ;
                    }
                    ;
                }
                ;
            }
            // LOGIN CLIENT
            this.client.once(discord_js_1.Events.ClientReady, (readyClient) => {
                this.log.info(`Logged in as ${readyClient.user.tag}`);
                // REGISTER COMMANDS
                if (this.commandsArray.length === 0)
                    return;
                const rest = new discord_js_1.REST().setToken(botToken);
                (() => __awaiter(this, void 0, void 0, function* () {
                    try {
                        this.log.info(`Started refreshing ${this.commandsArray.length} application (/) commands.`);
                        const data = yield rest.put(discord_js_1.Routes.applicationCommands(readyClient.user.id), { body: this.commandsArray });
                        this.log.info(`Successfully reloaded ${data.length} application (/) commands.`);
                    }
                    catch (error) {
                        this.log.error(`${error}`);
                    }
                    ;
                }))();
            });
            this.log.info("Logging in");
            this.client.login(botToken);
            // RECEIVE COMMANDS
            this.client.on(discord_js_1.Events.InteractionCreate, (interaction) => __awaiter(this, void 0, void 0, function* () {
                try {
                    if (interaction.isChatInputCommand()) {
                        const command = this.commands.get(interaction.commandName);
                        if (!command) {
                            this.log.error(`No command matching ${interaction.commandName} was found.`);
                            return;
                        }
                        ;
                        yield command.execute(interaction);
                        return;
                    }
                    ;
                }
                catch (error) {
                    this.log.error(`${error}`);
                    const errorMessage = { content: 'There was an error while executing this command!', flags: [discord_js_1.MessageFlags.Ephemeral] };
                    if (interaction.replied || interaction.deferred) {
                        yield interaction.followUp(errorMessage);
                    }
                    else {
                        yield interaction.reply(errorMessage);
                    }
                    ;
                }
                ;
                return;
            }));
        });
    }
}
exports.Bot = Bot;
exports.bot = new Bot();
